;------------------------------------------
;ASCII Finder 1.2
;by Andreas Finne
;a_finne@hotmail.com
;Copyright (C) 1999
;
;I have tried to comment everything.
;If you use anything from this source,
;please give me and anyone else who
;I credited in this source some credit.
;If you have any ideas, suggestions, comments
;or questions please e-mail me.
;If you find a way to optimize it, please tell me.
;------------------------------------------

#include "ti86asm.inc"

.org _asm_exec_ram

;------------------AShell / Rascall Description-------------------------
	nop		         	;Identifies the table
	jp Start			;Since it must be jp you can as well
						;take advantage of it
.dw $0000				;Version # of Table
.dw Title				;Absolute pointer to program description

					;These are here so I can use as many
					;jr's as possible. (jr's are 1 byte smaller than jp's)

Max:					
	ld a,236			;236 (max) into A
	jr DispEveryth
Min:					
	ld a,1				;1 (min) into A
	
DispEveryth:			;Routine to display current ASCII number and char
	ld hl,$0901			;Loads $09 into H and $01 into L
	ld (_curRow),hl		;Loads $01 into _curRow and $09 into _curCol
	ld hl,Empty			;Point HL to the text
	call _puts			;Put it out
	ld hl,$0803			;Same thing here
	ld (_curRow),hl		;This is done to get rid of the old ASCII number
	ld hl,Empty			;
	call _puts			;None of these destroys A
	ld (Nyt),a			;Load the current char into variable
	ld hl,10*256+48		;This is a little trick
	ld (_penCol),hl		;Loads 10 into _penRow and 48 into _penCol
	push af				;Save the value of the current char
	call DispA			;Routine to display A @ _penCol,_penRow, destroys A
	ld hl,$0703			; 
	ld (_curRow),hl		;Coordinates into _curRow and _curCol
	pop af				;Get the saved value back into A
	call _putmap		;ROM routine to display the char loaded into A
	cp 32				;If the number is 32 (Space)
	jr z,Space			;Then print 'space'
	cp 214				;Is it 214 (Enter)
	jr nz,KeyLoop		;If not, check keys again
	ld hl,24*256+50		;Number was 214
	ld (_penCol),hl		;24 into _penRow and 50 into _penCol
	ld hl,EnterTxt		;Point HL to the text
	call _vputs			;Put it out @ _penCol,_penRow in the small font
	jr KeyLoop			;Check keys again

Space:					;ASCII number was 32
	ld hl,24*256+50		;
	ld (_penCol),hl		;24 into _penRow and 50 into _penCol
	ld hl,SpaceTxt		;Point HL to the text
	call _vputs			;Put it out
	jr KeyLoop			;Check keys again


IncOne:					;Routine to increase to the next ASCII number
	ld a,(Nyt)			;Load current ASCII number into A
	cp 236				;Is it 236 (maximum)?
	jr z,KeyLoop		;If so, jump to check keys again
	inc a				;Otherwise, increase A
	jr DispEveryth		;Put everything out on the screen

DecOne:					;Routine to decrease to the previous ASCII number
	ld a,(Nyt)			;Load current ASCII number into A
	dec a				;Decrease it
	or a				;Did it become zero?
	jr z,KeyLoop		;If so, jump to check keys again
	jr DispEveryth		;Put everything out on the screen

IncTen:					;Routine to increase ASCII number with ten
	ld a,(Nyt)			;Load current ASCII number into A
	cp 227				;Compare to 227 (9 from maximum)
	jr nc,Max			;If number >= 227 call routine to make it maximum
	add a,10			;Add 10 to A
	jr DispEveryth		;Put everything out on the screen
	
DecTen:					;Routine to decrease ASCII number with ten
	ld a,(Nyt)			;Load current ASCII number into A
	cp 11				;Compare to 11 (10 from minimum)
	jr c,Min			;If number < 11 call routine to make it minimum
	sub 10				;Subtract 10 from A
	jr DispEveryth		;Put everything out on the screen

Start:        			;This is where it all begins
	call _runindicoff	;Turn off that little thing
	call _flushallmenus	;Remove all visible menus
	xor a				;Same as ld a,0
	ld ($C012),a		;Load that into length
	inc a				;Increase A (A=A+1)
	ld (Nyt),a			;Load A into variable
	jp ScreenSetup		;Call routine to put title and other text

KeyLoop:        		;Routine to wait for a key
	call _getkey		;Call a ROM routine that waits for a key to be pressed
	cp kRight			;A key has been pressed. Is it RIGHT?
	jr z,IncOne			;If so, jump to the routine to increase number by 1
	cp kLeft			;Is it LEFT?
	jr z,DecOne			;If so, jump to the routine to decrease number by 1
	cp kUp				;Is it UP?
	jr z,IncTen			;If so, jump to the routine to increase number by 10
	cp kDown			;Is it DOWN?
	jr z,DecTen			;If so, jump to the routine to decrease number by 10
	cp kExit			;Is it EXIT?
	jr z,Quit			;Then jump to exit routine
	cp kNext			;Is it MORE?
	jr z,View			;Then display the contents of the undel. mem.
	cp kF1				;Is it F1?
	jr z,Help 			;Then jump to help
	cp kEnter			;Is it ENTER?
	jr z,CopyChar		;Then copy the current character to undel. mem.
	cp kClear			;Is it CLEAR?
	jr z,Clear			;Then clear the undel. mem.
	jr KeyLoop			;None of these keys were pressed, check for keys again

Quit:					;Exit routine
	call _clrScrn		;If I use _clrLCD, everything isn't cleared
	jp _homeup			;Cursor into top left corner and return from program

Help:					;Display the help text
	call CEETAA			;Clear Everything Except Title And Author :)
	ld hl,17*256+1		;
	ld (_penCol),hl		;Display text
	ld hl,ClearTxt		;
	call _vputs			;
	ld bc,24*256+1		;I use bc instead of hl, because hl already points
	ld (_penCol),bc		;to the next text to print
	call _vputs			;	
	ld bc,31*256+1		;
	ld (_penCol),bc		;
	call _vputs			;
	ld bc,38*256+1		;
	ld (_penCol),bc		;
	call _vputs			;
	call _getkey		;Wait for a keypress
	jr ScreenSetup		;Reprint everything on the previous screen

CopyChar:				;Routine to copy char to undel. mem.
	ld hl,$C012			;$C012 contains the length of the undel. buffer
	ld a,(hl)			;Load a with length
	cp 84				;Is it 84 (maximum)?
	jr z,KeyLoop		;If so, don't copy any more, check keys again
	inc a				;Otherwise, increase A
	ld b,a				;Copy the new value into B
	ld (hl),b			;And load it into $C012
Loop:
	inc hl				;Increase HL one step
	djnz Loop			;Jump to 'Loop' B times. This makes HL point to the new place in undel. mem
	ld a,(Nyt)			;ASCII number into A
	ld (hl),a			;Load it in at the next place in the undel. mem
	jr KeyLoop			;Check keys again

View:					;Displays the contents of undel. mem.
	call CEETAA			; :)
	ld hl,$0815			;
	ld (_penCol),hl		;Display text
	ld hl,ContentsTxt	;
	call _vputs			;
	ld a,($C012)		;Load length into A
	or a				;Set flag if zero
	jr z,EmptyMem		;If zero flag is set then the mem is empty
	ld hl,$0002			;Begin to print it at 0,2
	ld (_curRow),hl		;
	ld hl,$C012			;Point HL to the length of the buffer, followed by the contents
	call _putps			;ROM routine that displays a string with leading length byte
	call _getkey		;Wait for a keypress
	jr ScreenSetup		;Reprint everything on the previous screen

Clear:					;Clear the undel mem
	call CEETAA			;
	ld hl,$0B01			;
	ld (_penCol),hl		;
	ld hl,ClearQTxt		;Print text
	call _vputs			;
	ld bc,$0404			;I use bc because hl points to the next text
	ld (_curRow),bc		;
	call _puts			;Put it out
	ld a,$0D			;
	ld (_curCol),a		;New X-Coord for text
	call _puts			;Put it out
ClearLoop:
	call _getkey		;Wait for a key to be pressed
	cp kDown			;Is it DOWN?
	jr z,GoBack			;If so then don't clear the undel mem
	cp kUp				;Check if it is UP
	jr nz,ClearLoop		;If not, jump to ClearLoop
	xor a				;Same as ld a,0
	ld ($C012),a		;Load that into length
	call CEETAA			;
EmptyMem:
	ld hl,$0804			;
	ld (_curRow),hl		;
	ld hl,EmptyTxt		;Displays the text 'Empty!'
	call _puts			;
	call _getkey		;Wait for a keypress
GoBack:
ScreenSetup:
	call _clrLCD		;Clear the screen
	call _homeup		;Cursor in upper left corner
	set textInverse,(iy+textflags)	;White text on black
	ld hl,Title
	call _puts			;
	ld bc,$0005			;Display text
	ld (_curRow),bc		;
	call _puts			;	
	res textInverse,(iy+textflags)	;Back to normal mode (black on white)
	ld bc,10*256+1
	ld (_penCol),bc
	call _vputs
	ld bc,24*256+1
	ld (_penCol),bc
	call _vputs
	ld bc,32*256+1
	ld (_penCol),bc
	call _vputs			;More text
	ld bc,51*256+27
	ld (_penCol),bc
	call _vputs
	ld bc,57*256+27
	ld (_penCol),bc
	call _vputs
	ld a,(Nyt)			;Load a with the ASCII number
	jp DispEveryth		;Display the number and the char

CEETAA:                 ;Clear Everything Except Title And Author :)
	ld hl,$FCA0			;Address to clear pixel before the ones to be cleared
	ld de,$FCA0+1		;Next pixel
	ld bc,$025F        	;607 pixels to be cleared
	ldir				;ld (de),(hl)/inc de/inc hl, bc times
	ret					;Return from call

DispA:                                  ;Routine to Display A [By: Matthew Shepcar]
 ld l,a                                 ;Loads a into l
 ld h,0                                 ;0 into H
DispHL:                                 ;Displays HL [By: Matthew Shepcar]
 xor a                                  ;Loads 0 into A
 ld de,-1                               ;Loads -1 into DE
 ld (_curRow),de                        ;Loads de into CurRow
 call 4A33h                             ;Calls 4A33h
 dec hl                                 ;Decrease it
 jp _vputs                              ;Put on screen and return


;****** Start of text ******
;The texts must be in this order
;or else hl points to the wrong text

Empty:
	.db "     ",0

Title:
	.db "  ASCII Finder 1.2   ",0

ChooseTxt:
	.db " ",207,5," +/- 1"
	.db "  ",6,7," +/- 10 ",0

Text:
	.db "ASCII Number:",0

Chartxt:
	.db "Character:",0

HelpTxt:
	.db "Press F1 for help!",0

Author:
	.db "Made by Andreas Finne",0
Email:
	.db "a_finne@hotmail.com",0

SpaceTxt:
	.db "<SPACE>",0

EnterTxt:
	.db "<ENTER>",0

ContentsTxt:
	.db "Contents of undel. mem:",0

ClearTxt:
	.db "Clear: Clear undel. mem",0

CopyTxt:
	.db "Enter: Copy character to"
	.db " undel. mem",0
ViewTxt:
	.db "More: View contents of"
	.db " undel. mem",0

QuitTxt:
	.db "Exit: Quit",0

ClearQTxt:
	.db "Do you want"
	.db " to clear the undel. mem?",0
Yes:    .db 6," Yes",0
No:     .db 7," No",0

EmptyTxt:
	.db "Empty!",0

Nyt:				;Contains the ASCII number
	.db 0

.end
